function img=PETreconstruction(P,R,D)
% =========================================================================
% funkce PETreconstruction slouzici k rekonstrukci surovych PET dat
% ziskanych z realneho mereni nebo simulaci
% img = PETreconstruction(P,R,D);
% vystupem je 'img' - rekonstruovanz obrazek na zaklade zadanych dat
% vstupem jsou:
%    P je symetricka matice detekkonaych demitovanych paru fotonu o
%    velikosti [D x D] 
%    R je polomer detekcniho prstence pouziteho pro mereni nebo simulaci
%    (velikost je v px) 
%    D je pocet detektoru na detekcnim prstenci
% =========================================================================

 angDet=0:1/(D):1;
 angDet=angDet(1:end-1)*2*pi+pi/D;
 
 r=floor(R);
 radons=[D,r*2+1];
 [d,theta]=meshgrid(-r:r,angDet);
 d=d(:)'; theta=theta(:)';
 a=cos(theta); b=sin(theta); c=-d;
 
 [ang1,ang2]=lA(a,b,c,R);
 ang1=reshape(ang1,radons);
 ang2=reshape(ang2,radons);
 
 detpairpad=padarray(P,[1,1],'circular','both');
 radonsp=interp2([-angDet(1),angDet,angDet(end)+2*angDet(1)],[-angDet(1),angDet,angDet(end)+2*angDet(1)],detpairpad,ang1,ang2,'spline',0)';
 img=iradon(radonsp,angDet/2/pi*360);
return;

function [ang1,ang2]=lA(a,b,c,r)
 ind=abs(a)>1/2;
 D(ind)=(2*b(ind).*c(ind)./a(ind).^2).^2-4*(b(ind).^2./a(ind).^2+1).*(c(ind).^2./a(ind).^2-r^2);
 D(~ind)=(2*a(~ind).*c(~ind)./b(~ind).^2).^2-4*(a(~ind).^2./b(~ind).^2+1).*(c(~ind).^2./b(~ind).^2-r^2);
 if any(D<0), error('zaporny diskriminant, to by se stat nemelo'); end;
 i1(2,ind)=(-2.*b(ind).*c(ind)./a(ind).^2+sqrt(D(ind)))./(2.*b(ind).^2./a(ind).^2+2);
 i1(1,~ind)=(-2.*a(~ind).*c(~ind)./b(~ind).^2+sqrt(D(~ind)))./(2.*a(~ind).^2./b(~ind).^2+2);
 i2(2,ind)=(-2.*b(ind).*c(ind)./a(ind).^2-sqrt(D(ind)))./(2.*b(ind).^2./a(ind).^2+2);
 i2(1,~ind)=(-2.*a(~ind).*c(~ind)./b(~ind).^2-sqrt(D(~ind)))./(2.*a(~ind).^2./b(~ind).^2+2);
 i1(1,ind)=-(b(ind).*i1(2,ind)+c(ind))./a(ind); i1(2,~ind)=-(a(~ind).*i1(1,~ind)+c(~ind))./b(~ind);
 i2(1,ind)=-(b(ind).*i2(2,ind)+c(ind))./a(ind); i2(2,~ind)=-(a(~ind).*i2(1,~ind)+c(~ind))./b(~ind);
 ang1=atan(i1(2,:)./i1(1,:)); ang2=atan(i2(2,:)./i2(1,:));
 ang1(i1(2,:)<0&i1(1,:)<0)=ang1(i1(2,:)<0&i1(1,:)<0)+pi; ang2(i2(2,:)<0&i2(1,:)<0)=ang2(i2(2,:)<0&i2(1,:)<0)+pi;
 ang1(i1(2,:)>0&i1(1,:)<0)=ang1(i1(2,:)>0&i1(1,:)<0)+pi; ang2(i2(2,:)>0&i2(1,:)<0)=ang2(i2(2,:)>0&i2(1,:)<0)+pi;
 ang1(ang1<0)=ang1(ang1<0)+2*pi; ang2(ang2<0)=ang2(ang2<0)+2*pi;
return;