function [ang1,ang2]=lineAndCircle2intersectionAngles(a,b,c,r,stred,fantom)
% =========================================================================
% [ang1,ang2]=lineAndCircle2intersectionAngles(a,b,c,r,stred,fantom)
% 
% funkce spocita pruseciky primky zadane parametry a,b,c se zvolenou
% kruznici zadanouo stredem a polomerem; vystupem jsou dvojice uhlu kde
% primka protla kruznici
% 
% PARAMETRY:
% * a,b,c - vektor parametru primky A*x+B*y+C=0, velikost je [1 N]
% * r - polomer kruznice na kterem jsouo umisteny senzory a kde jsou
%       detekovany vyrazene castice
% * stred - stred fantomu vzhledem obrazku fantomu, velikost je [1 2]
% * fantom - obrazek fantomu (volitelny), pokud je zadan vykresli se
%            relevantni visualizace simulace, velikost je [X Y]
% 
% VYSTUP:
% * ang1,ang2 - dvojice uhlu na kruznici kde na emitovana castice protla
%               kruznici detektoru
% 
% PRIKLAD:
% 
% -------------------------------------------------------------------------

if nargin<5,  error('too few parameters');  end;

 if exist('fantom','var')
    pcar=300;
    figure;
    imagesc(-stred+1:stred-1,-stred+1:stred-1,-fantom); colormap gray; hold on;
    for i=1:max([round(length(c)/pcar),1]):length(c), plot([-r,r],-(-a(i)*[-r,r]-c(i))/b(i),'r');end;
    plot(-r:r,-sqrt(r^2-(-r:r).^2),'g');
    plot(-r:r,sqrt(r^2-(-r:r).^2),'g');
    axis([-r,r,-r,r]);
 end;

 ind=abs(a)>1/2;
 D(ind)=(2*b(ind).*c(ind)./a(ind).^2).^2-4*(b(ind).^2./a(ind).^2+1).*(c(ind).^2./a(ind).^2-r^2);
 D(~ind)=(2*a(~ind).*c(~ind)./b(~ind).^2).^2-4*(a(~ind).^2./b(~ind).^2+1).*(c(~ind).^2./b(~ind).^2-r^2);

 if any(D<0), error('zaporny diskriminant, to by se stat nemelo'); end;
 i1(2,ind)=(-2.*b(ind).*c(ind)./a(ind).^2+sqrt(D(ind)))./(2.*b(ind).^2./a(ind).^2+2);
 i1(1,~ind)=(-2.*a(~ind).*c(~ind)./b(~ind).^2+sqrt(D(~ind)))./(2.*a(~ind).^2./b(~ind).^2+2);

 i2(2,ind)=(-2.*b(ind).*c(ind)./a(ind).^2-sqrt(D(ind)))./(2.*b(ind).^2./a(ind).^2+2);
 i2(1,~ind)=(-2.*a(~ind).*c(~ind)./b(~ind).^2-sqrt(D(~ind)))./(2.*a(~ind).^2./b(~ind).^2+2);

 i1(1,ind)=-(b(ind).*i1(2,ind)+c(ind))./a(ind);
 i1(2,~ind)=-(a(~ind).*i1(1,~ind)+c(~ind))./b(~ind);

 i2(1,ind)=-(b(ind).*i2(2,ind)+c(ind))./a(ind);
 i2(2,~ind)=-(a(~ind).*i2(1,~ind)+c(~ind))./b(~ind);

 if exist('fantom','var')
    plot(i1(1,1:max([round(length(c)/pcar),1]):length(c)),-i1(2,1:max([round(length(c)/pcar),1]):length(c)),'bx');
    plot(i2(1,1:max([round(length(c)/pcar),1]):length(c)),-i2(2,1:max([round(length(c)/pcar),1]):length(c)),'b+');
 end;
 
 ang1=atan(i1(2,:)./i1(1,:));
 ang2=atan(i2(2,:)./i2(1,:));
 ang1(i1(2,:)<0&i1(1,:)<0)=ang1(i1(2,:)<0&i1(1,:)<0)+pi;
 ang2(i2(2,:)<0&i2(1,:)<0)=ang2(i2(2,:)<0&i2(1,:)<0)+pi;
 ang1(i1(2,:)>0&i1(1,:)<0)=ang1(i1(2,:)>0&i1(1,:)<0)+pi;
 ang2(i2(2,:)>0&i2(1,:)<0)=ang2(i2(2,:)>0&i2(1,:)<0)+pi;
 ang1(ang1<0)=ang1(ang1<0)+2*pi;
 ang2(ang2<0)=ang2(ang2<0)+2*pi;

 if exist('fantom','var')
    plot(i1(1,1:max([round(length(c)/pcar),1]):length(c)),-tan(ang1(1:max([round(length(c)/pcar),1]):length(c))).*i1(1,1:max([round(length(c)/pcar),1]):length(c)),'go');
    plot(i2(1,1:max([round(length(c)/pcar),1]):length(c)),-tan(ang2(1:max([round(length(c)/pcar),1]):length(c))).*i2(1,1:max([round(length(c)/pcar),1]):length(c)),'go');
 end;
return;